package de.devbliss.paulparser;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * A container class that constructs a clean and nice PublicationModel from a 
 * PublicationModelRaw.
 * 
 * More information right at the constructor.
 * 
 * In general we follow:
 * http://scholar.google.com/intl/en/scholar/inclusion.html
 * 
 * 
 */
public class PublicationModel {

	public List<String> authors;
	public String title;

	public Date date;
	// when this paper is a journal:
	public String journal_title;
	public String doi;
	
	public String description;

	
	/**
	 * PublicationModelRaw usually comes from the PublicationMetaTagParser and contains
	 * a lot of duplicate and raw information.
	 * 
	 * If you construct the PublicationModel via the constructor all
	 * data of PublicationModelRaw is integrated in a clean way and de-duplicated where
	 * possible.
	 * 
	 * You end up having a "nice" and "clean" usable object of the publication metadata.
	 * 
	 * In general we follow:
	 * http://scholar.google.com/intl/en/scholar/inclusion.html
	 * 
	 * @param publicationModelRaw
	 */
	public PublicationModel(
			PublicationModelRaw publicationModelRaw) {

		authors = new ArrayList<String>();

		// extract authors from raw model:
		doIntegrateAuthors(publicationModelRaw);

		// extract title from raw model:
		doIntegrateTitle(publicationModelRaw);
		
		doIntegrateConferenceTechnicalReportOrDissertation(publicationModelRaw);

		doIntegrateDoi(publicationModelRaw);
		
		doIntegrateDate(publicationModelRaw);
		
		doIntegrateDescription(publicationModelRaw);


	}
	
	/**
	 * not sure why you want to use that... except you are a unit test...
	 */
	public PublicationModel () {
		authors = new ArrayList<String>();		
	}

	private void doIntegrateAuthors(
			PublicationModelRaw publicationParsingMetaModel) {

		// first check citation_
		if (publicationParsingMetaModel.citation_author.size() > 0) {

			for (String author : publicationParsingMetaModel.citation_author) {

				authors.add(author);
			}

		// then check dc creators:
		} else if (publicationParsingMetaModel.dcCreator.size() > 0) {

			for (String author : publicationParsingMetaModel.dcCreator) {

				authors.add(author);
			}


		// citation_authors is usually a line with ";" as separator like so
		// "first author; second author; third author"
		} else if (publicationParsingMetaModel.citation_authors != null) {

			// split them. All journals we know of use ";" as separator
			String[] authorSplit = publicationParsingMetaModel.citation_authors
					.split(";");

			if (authorSplit.length > 0) {

				for (String author : authorSplit) {
					//add and dont't forget to trim :)
					authors.add(author.trim());
				}

			}

		// then check contributors (that might be wrong, but we accept it
		// anyways as some journals are using that.)
		} else if (publicationParsingMetaModel.dcContributor.size() > 0) {

			for (String author : publicationParsingMetaModel.dcContributor) {

				authors.add(author);
			}

		}

	}

	private void doIntegrateTitle(
			PublicationModelRaw publicationParsingMetaModel) {

		if (publicationParsingMetaModel.citation_title != null) {
			title = publicationParsingMetaModel.citation_title;

		} else if (publicationParsingMetaModel.dcTitle != null) {
			title = publicationParsingMetaModel.dcTitle;

		}

	}

	private void doIntegrateDate(
			PublicationModelRaw publicationModelRaw) {

		if (publicationModelRaw.citation_date != null) {
			date = FuzzyDateParser.parse(publicationModelRaw.citation_date);			
		} 
		
		if (date == null && publicationModelRaw.citation_publication_date != null) {
			date = FuzzyDateParser.parse(publicationModelRaw.citation_publication_date);			
		} 
		
		if (date == null && publicationModelRaw.dcIssued != null) {
			date = FuzzyDateParser.parse(publicationModelRaw.dcIssued);			
		} 
		
		if (date == null && publicationModelRaw.dcDate != null) {
			date = FuzzyDateParser.parse(publicationModelRaw.dcDate);
		}

	}

	private void doIntegrateConferenceTechnicalReportOrDissertation(
			PublicationModelRaw publicationParsingMetaModel) {

		if (publicationParsingMetaModel.citation_journal_title != null) {
			
			journal_title = publicationParsingMetaModel.citation_journal_title;

		} 

	}

	private void doIntegrateDoi(PublicationModelRaw publicationParsingMetaModel) {

		String tempDoi = null;
		
		if (publicationParsingMetaModel.citation_doi != null) {
			tempDoi = publicationParsingMetaModel.citation_doi;
			
		// Note: A "dc.identifier" must not necessarily be a doi.
		// But in our scientific community it is in almost all cases we know of.
		} else if (publicationParsingMetaModel.dcIdentifier != null) {
			tempDoi = publicationParsingMetaModel.dcIdentifier;
		}
		
		
		// Don't forget to clean doi...
		// ...sometimes dois start with "doi:"
		// that is fine, but by convention we remove it.
		if (tempDoi != null) {
			if (tempDoi.startsWith("doi:")) {
				tempDoi = tempDoi.substring(4, tempDoi.length());
			}
			doi = tempDoi;
		}
		
		
	}
	
	private void doIntegrateDescription(PublicationModelRaw publicationParsingMetaModel){
		
		if (publicationParsingMetaModel.dcDescription != null) {
			
			description = publicationParsingMetaModel.dcDescription;
		}
		
		
	}

	/**
	 * Convenience method that follows guidelines from
	 * http://scholar.google.com/intl/en/scholar/inclusion.html
	 * 
	 * Therefore we say "yes" when at least the following criteria can be
	 * deducted: 
	 * - authors 
	 * - title
	 * - publication_date
	 * 
	 */
	boolean isValidCitation() {

		if (title != null && journal_title != null && authors.size() > 0) {

			return true;
			
		} else {

			return false;
		}

	}
}
